@@ -30,6 +30,6 @@ ideff <- alpha[paste('id',id,sep='.'),'effect']
 firmeff <- alpha[paste('firm',firm,sep='.'),'effect']
 
 ## verify that id and firm coefficients are 1
-cat('accuracy:',coef(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + ideff + firmeff-1))[7:8]-1,'\n')
+cat('accuracy:',sprintf('%.8e',coef(lm(y ~ x + x2 + x3 + x4 + x5 + x6 + ideff + firmeff-1))[7:8],'\n'))
 
 
